@@ -3,8 +3,10 @@ import json
 import logging
 from datetime import datetime
 from enum import Enum as EnumType
-from typing import Any, Dict, List, Optional, Tuple
+from typing import Any, Dict, List, Optional
 
+from celery.result import AsyncResult
+from pydantic import root_validator
 from sqlalchemy import Column, DateTime
 from sqlalchemy import Enum as EnumColumn
 from sqlalchemy import ForeignKey, String
@@ -27,6 +29,7 @@ from fidesops.models.policy import (
     WebhookDirection,
     WebhookTypes,
 )
+from fidesops.schemas.base_class import BaseSchema
 from fidesops.schemas.drp_privacy_request import DrpPrivacyRequestCreate
 from fidesops.schemas.external_https import (
     SecondPartyRequestFormat,
@@ -35,9 +38,11 @@ from fidesops.schemas.external_https import (
 )
 from fidesops.schemas.masking.masking_secrets import MaskingSecretCache
 from fidesops.schemas.redis_cache import PrivacyRequestIdentity
+from fidesops.tasks import celery_app
 from fidesops.util.cache import (
     FidesopsRedis,
     get_all_cache_keys_for_privacy_request,
+    get_async_task_tracking_cache_key,
     get_cache,
     get_drp_request_body_cache_key,
     get_encryption_cache_key,
@@ -50,6 +55,43 @@ from fidesops.util.oauth_util import generate_jwe
 logger = logging.getLogger(__name__)
 
 
+class ManualAction(BaseSchema):
+    """Surface how to manually retrieve or mask data in a database-agnostic way
+
+    "locators" are similar to the SQL "WHERE" information.
+    "get" contains a list of fields that should be retrieved from the source
+    "update" is a dictionary of fields and the value they should be replaced with.
+    """
+
+    locators: Dict[str, Any]
+    get: Optional[List[str]]
+    update: Optional[Dict[str, Any]]
+
+    @root_validator
+    @classmethod
+    def get_or_update_details(
+        cls: BaseSchema, values: Dict[str, Any]
+    ) -> Dict[str, Any]:
+        """Validate that 'get' or 'update' details are supplied."""
+        if values and not values.get("get") and not values.get("update"):
+            raise ValueError(
+                "A ManualAction requires either 'get' or 'update' instructions."
+            )
+
+        return values
+
+
+class StoppedCollection(BaseSchema):
+    """Class that contains details about the collection that halted privacy request execution"""
+
+    step: PausedStep
+    collection: CollectionAddress
+    action_needed: Optional[List[ManualAction]] = None
+
+    class Config:
+        arbitrary_types_allowed = True
+
+
 class PrivacyRequestStatus(str, EnumType):
     """Enum for privacy request statuses, reflecting where they are in the Privacy Request Lifecycle"""
 
@@ -59,6 +101,7 @@ class PrivacyRequestStatus(str, EnumType):
     in_processing = "in_processing"
     complete = "complete"
     paused = "paused"
+    canceled = "canceled"
     error = "error"
 
 
@@ -72,7 +115,7 @@ def generate_request_callback_jwe(webhook: PolicyPreWebhook) -> str:
     return generate_jwe(json.dumps(jwe.dict()))
 
 
-class PrivacyRequest(Base):
+class PrivacyRequest(Base):  # pylint: disable=R0904
     """
     The DB ORM model to describe current and historic PrivacyRequests. A privacy request is a
     database record representing a data subject request's progression within the Fidesops system.
@@ -117,6 +160,9 @@ class PrivacyRequest(Base):
         backref="privacy_requests",
     )
 
+    cancel_reason = Column(String(200))
+    canceled_at = Column(DateTime(timezone=True), nullable=True)
+
     # passive_deletes="all" prevents execution logs from having their privacy_request_id set to null when
     # a privacy_request is deleted.  We want to retain for record-keeping.
     execution_logs = relationship(
@@ -138,8 +184,9 @@ class PrivacyRequest(Base):
     )
 
     reviewer = relationship(
-        "FidesopsUser", backref=backref("privacy_request", passive_deletes=True)
+        FidesopsUser, backref=backref("privacy_request", passive_deletes=True)
     )
+    paused_at = Column(DateTime(timezone=True), nullable=True)
 
     @classmethod
     def create(cls, db: Session, *, data: Dict[str, Any]) -> FidesopsBase:
@@ -173,6 +220,26 @@ class PrivacyRequest(Base):
                     value,
                 )
 
+    def cache_task_id(self, task_id: str) -> None:
+        """Sets a task_id for this privacy request's asynchronous execution."""
+        cache: FidesopsRedis = get_cache()
+        cache.set(
+            get_async_task_tracking_cache_key(self.id),
+            task_id,
+        )
+
+    def get_cached_task_id(self) -> Optional[str]:
+        """Gets the cached task ID for this privacy request."""
+        cache: FidesopsRedis = get_cache()
+        task_id = cache.get(get_async_task_tracking_cache_key(self.id))
+        return task_id
+
+    def get_async_execution_task(self) -> Optional[AsyncResult]:
+        """Returns a task reflecting the state of this privacy request's asynchronous execution."""
+        task_id = self.get_cached_task_id()
+        res: AsyncResult = AsyncResult(task_id)
+        return res
+
     def cache_drp_request_body(self, drp_request_body: DrpPrivacyRequestCreate) -> None:
         """Sets the identity's values at their specific locations in the Fidesops app cache"""
         cache: FidesopsRedis = get_cache()
@@ -229,51 +296,58 @@ class PrivacyRequest(Base):
         result_prefix = f"{self.id}__*"
         return cache.get_encoded_objects_by_prefix(result_prefix)
 
-    PAUSED_SEPARATOR = "__fidesops_paused_sep__"
-
-    def cache_paused_step_and_collection(
+    def cache_paused_collection_details(
         self,
-        paused_step: Optional[PausedStep] = None,
-        paused_collection: Optional[CollectionAddress] = None,
+        step: Optional[PausedStep] = None,
+        collection: Optional[CollectionAddress] = None,
+        action_needed: Optional[List[ManualAction]] = None,
     ) -> None:
         """
-        When a privacy request is paused, cache both the paused step (access or erasure) and the collection
-        awaiting manual input.  For example, we might pause a privacy request at the erasure step of the
-        postgres_example:address collection.
-
-        The paused_step is needed because we may build and execute multiple graphs as part of running a privacy request.
-        An erasure request builds two graphs, one to access the data, and the second to mask it.
-        We need to know if we should resume execution from the access or the erasure portion of the request.
+        Cache details about the paused step, paused collection, and any action needed to resume the privacy request.
         """
-        cache: FidesopsRedis = get_cache()
-        paused_key = f"PAUSED_LOCATION__{self.id}"
-
-        # Store both the paused separator and paused collection in one value
-        cache.set_encoded_object(
-            paused_key,
-            f"{paused_step.value}{self.PAUSED_SEPARATOR}{paused_collection.value}"
-            if paused_step and paused_collection
-            else None,
+        cache_restart_details(
+            cache_key=f"PAUSED_LOCATION__{self.id}",
+            step=step,
+            collection=collection,
+            action_needed=action_needed,
         )
 
-    def get_paused_step_and_collection(
+    def get_paused_collection_details(
         self,
-    ) -> Tuple[Optional[PausedStep], Optional[CollectionAddress]]:
-        """Get both the paused step (access or erasure) and collection awaiting manual input for the given privacy request.
+    ) -> Optional[StoppedCollection]:
+        """Return details about the paused step, paused collection, and any action needed to resume the paused privacy request.
 
         The paused step lets us know if we should resume privacy request execution from the "access" or the "erasure"
         portion of the privacy request flow, and the collection tells us where we should cache manual input data for later use,
         In other words, this manual data belongs to this collection.
         """
-        cache: FidesopsRedis = get_cache()
-        node_addr: str = cache.get_encoded_by_key(f"EN_PAUSED_LOCATION__{self.id}")
+        return get_restart_details(cached_key=f"EN_PAUSED_LOCATION__{self.id}")
 
-        if node_addr:
-            split_addr = node_addr.split(self.PAUSED_SEPARATOR)
-            return PausedStep(split_addr[0]), CollectionAddress.from_string(
-                split_addr[1]
-            )
-        return None, None  # If no cached data, return a tuple of Nones
+    def cache_failed_collection_details(
+        self,
+        step: Optional[PausedStep] = None,
+        collection: Optional[CollectionAddress] = None,
+    ) -> None:
+        """
+        Cache details about the failed step and failed collection details. No specific input data is required to resume
+        a failed request, so action_needed is None.
+        """
+        cache_restart_details(
+            cache_key=f"FAILED_LOCATION__{self.id}",
+            step=step,
+            collection=collection,
+            action_needed=None,
+        )
+
+    def get_failed_collection_details(
+        self,
+    ) -> Optional[StoppedCollection]:
+        """Get details about the failed step (access or erasure) and collection that triggered failure.
+
+        The failed step lets us know if we should resume privacy request execution from the "access" or the "erasure"
+        portion of the privacy request flow.
+        """
+        return get_restart_details(cached_key=f"EN_FAILED_LOCATION__{self.id}")
 
     def cache_manual_input(
         self, collection: CollectionAddress, manual_rows: Optional[List[Row]]
@@ -378,7 +452,33 @@ class PrivacyRequest(Base):
         """Dispatches this PrivacyRequest throughout the Fidesops System"""
         if self.started_processing_at is None:
             self.started_processing_at = datetime.utcnow()
-            self.save(db=db)
+        if self.status == PrivacyRequestStatus.pending:
+            self.status = PrivacyRequestStatus.in_processing
+        self.save(db=db)
+
+    def pause_processing(self, db: Session) -> None:
+        """Mark privacy request as paused, and save paused_at"""
+        self.update(
+            db,
+            data={
+                "status": PrivacyRequestStatus.paused,
+                "paused_at": datetime.utcnow(),
+            },
+        )
+
+    def cancel_processing(self, db: Session, cancel_reason: Optional[str]) -> None:
+        """Cancels a privacy request.  Currently should only cancel 'pending' tasks"""
+        if self.canceled_at is None:
+            self.status = PrivacyRequestStatus.canceled
+            self.cancel_reason = cancel_reason
+            self.canceled_at = datetime.utcnow()
+            self.save(db)
+
+            task_id = self.get_cached_task_id()
+            if task_id:
+                logger.info(f"Revoking task {task_id} for request {self.id}")
+                # Only revokes if execution is not already in progress
+                celery_app.control.revoke(task_id, terminate=False)
 
     def error_processing(self, db: Session) -> None:
         """Mark privacy request as errored, and note time processing was finished"""
@@ -391,6 +491,53 @@ class PrivacyRequest(Base):
         )
 
 
+# Unique text to separate a step from a collection address, so we can store two values in one.
+PAUSED_SEPARATOR = "__fidesops_paused_sep__"
+
+
+def cache_restart_details(
+    cache_key: str,
+    step: Optional[PausedStep] = None,
+    collection: Optional[CollectionAddress] = None,
+    action_needed: Optional[List[ManualAction]] = None,
+) -> None:
+    """Generic method to cache which collection and step(access/erasure) halted PrivacyRequest execution,
+    as well as data (action_needed) to resume the privacy request if applicable.
+
+    For example, we might pause a privacy request at the access step of the postgres_example:address collection.  The
+    user might need to retrieve an "email" field and an "address" field where the customer_id is 22 to resume the request.
+
+    The "step" is needed because we may build and execute multiple graphs as part of running a privacy request.
+    An erasure request builds two graphs, one to access the data, and the second to mask it.
+    We need to know if we should resume execution from the access or the erasure portion of the request.
+    """
+    cache: FidesopsRedis = get_cache()
+    cache_stopped: Optional[StoppedCollection] = None
+    if collection and step:
+        cache_stopped = StoppedCollection(
+            step=step, collection=collection, action_needed=action_needed
+        )
+
+    cache.set_encoded_object(
+        cache_key,
+        cache_stopped.dict() if cache_stopped else None,
+    )
+
+
+def get_restart_details(
+    cached_key: str,
+) -> Optional[StoppedCollection]:
+    """Get details about the collection that halted privacy request execution and details to resume if applicable.
+
+    The "step" lets us know if we should resume privacy request execution from the "access" or the "erasure"
+    portion of the privacy request flow, and the collection lets us know which node we stopped on.  "action_needed"
+    describes actions that need to be manually performed before resuming request.
+    """
+    cache: FidesopsRedis = get_cache()
+    cached_stopped: Optional[StoppedCollection] = cache.get_encoded_by_key(cached_key)
+    return StoppedCollection.parse_obj(cached_stopped) if cached_stopped else None
+
+
 class ExecutionLogStatus(EnumType):
     """Enum for execution log statuses, reflecting where they are in their workflow"""
 
@@ -400,6 +547,7 @@ class ExecutionLogStatus(EnumType):
     error = "error"
     paused = "paused"
     retrying = "retrying"
+    skipped = "skipped"
 
 
 class ExecutionLog(Base):
